@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdhisteq.c,v 1.31 2009/05/13 21:06:42 guru Exp $
+ *	$Id: grdhisteq.c,v 1.37 2010/03/22 21:51:29 jluis Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -45,22 +45,22 @@
 
 struct GRDHISTEQ_CTRL {
 	struct C {	/* -C<n_cells>*/
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG value;
 	} C;
 	struct D {	/* -D */
-		BOOLEAN active;
+		GMT_LONG active;
 	} D;
 	struct G {	/* -G<file> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct N {	/* -N[<norm>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double norm;
 	} N;
 	struct Q {	/* -Q */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Q;
 };
 
@@ -76,7 +76,7 @@ struct	CELL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN	error = FALSE;
+	GMT_LONG	error = FALSE;
 	
 	char *infile = CNULL;
 
@@ -84,13 +84,13 @@ int main (int argc, char **argv)
 		
 	struct GRDHISTEQ_CTRL *Ctrl;
 
-	void do_usual (char *infile, char *outfile, GMT_LONG n_cells, BOOLEAN quadratic, BOOLEAN dump_intervals, int argc, char **argv);
+	void do_usual (char *infile, char *outfile, GMT_LONG n_cells, GMT_LONG quadratic, GMT_LONG dump_intervals, int argc, char **argv);
 	void do_gaussian (char *infile, char *outfile, double norm, int argc, char **argv);
-	void *New_Grdhisteq_Ctrl (), Free_Grdhisteq_Ctrl (struct GRDHISTEQ_CTRL *C);
+	void *New_grdhisteq_Ctrl (), Free_grdhisteq_Ctrl (struct GRDHISTEQ_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDHISTEQ_CTRL *)New_Grdhisteq_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDHISTEQ_CTRL *)New_grdhisteq_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -171,14 +171,14 @@ int main (int argc, char **argv)
 	else
 		do_usual (infile, Ctrl->G.file, Ctrl->C.value, Ctrl->Q.active, Ctrl->D.active, argc, argv);
 
-	Free_Grdhisteq_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdhisteq_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void do_usual (char *infile, char *outfile, GMT_LONG n_cells, BOOLEAN quadratic, BOOLEAN dump_intervals, int argc, char **argv)
+void do_usual (char *infile, char *outfile, GMT_LONG n_cells, GMT_LONG quadratic, GMT_LONG dump_intervals, int argc, char **argv)
 {
 	char format[BUFSIZ];
 	GMT_LONG	last_cell, n_cells_m1 = 0, current_cell, i, j, nxy, nxy_0;
@@ -232,7 +232,7 @@ void do_usual (char *infile, char *outfile, GMT_LONG n_cells, BOOLEAN quadratic,
 		cell[current_cell].low = data[i];
 		cell[current_cell].high = data[j];
 
-		if (dump_intervals) fprintf (GMT_stdout, format, data[i], data[j], current_cell);
+		if (dump_intervals) fprintf (stdout, format, data[i], data[j], current_cell);
 
 		i = j;
 		current_cell++;
@@ -359,16 +359,16 @@ int compare_indices (const void *point_1, const void *point_2)
 		return (0);
 }
 
-void *New_Grdhisteq_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdhisteq_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDHISTEQ_CTRL *C;
 	
-	C = (struct GRDHISTEQ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDHISTEQ_CTRL), "New_Grdhisteq_Ctrl");
+	C = (struct GRDHISTEQ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDHISTEQ_CTRL), "New_grdhisteq_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	return ((void *)C);
 }
 
-void Free_Grdhisteq_Ctrl (struct GRDHISTEQ_CTRL *C) {	/* Deallocate control structure */
+void Free_grdhisteq_Ctrl (struct GRDHISTEQ_CTRL *C) {	/* Deallocate control structure */
 	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
