@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdmask.c,v 1.78 2009/09/09 23:27:03 guru Exp $
+ *	$Id: grdmask.c,v 1.86 2010/08/19 18:43:33 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -42,27 +42,27 @@
 
 struct GRDMASK_CTRL {
 	struct A {	/* -A[m|p|step] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;
 		double step;
 	} A;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<maskfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N<maskvalues> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double mask[GRDMASK_N_CLASSES];	/* values for each level */
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
@@ -70,7 +70,7 @@ struct GRDMASK_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN	error = FALSE, done, nofile = TRUE, periodic = FALSE, resample = FALSE;
+	GMT_LONG	error = FALSE, done, nofile = TRUE, periodic = FALSE, resample = FALSE;
 
 	char line[BUFSIZ], ptr[BUFSIZ], *not_used = NULL;
 
@@ -91,11 +91,11 @@ int main (int argc, char **argv)
 	struct GMT_LINE_SEGMENT P;
 	struct GRDMASK_CTRL *Ctrl;
 	
-	void *New_Grdmask_Ctrl (), Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C);
+	void *New_grdmask_Ctrl (), Free_grdmask_Ctrl (struct GRDMASK_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct GRDMASK_CTRL *)New_Grdmask_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct GRDMASK_CTRL *)New_grdmask_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	GMT_grd_init (&header, argc, argv, FALSE);
 
@@ -247,7 +247,7 @@ int main (int argc, char **argv)
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
 
-	header.node_offset = Ctrl->F.active;
+	header.node_offset = (int)Ctrl->F.active;
 	header.x_inc = Ctrl->I.xinc;
 	header.y_inc = Ctrl->I.yinc;
 	GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
@@ -283,7 +283,7 @@ int main (int argc, char **argv)
 		dj = (GMT_LONG)ceil (Ctrl->S.radius * idy);
 	}
 	periodic = (GMT_io.in_col_type[GMT_X] == GMT_IS_LON);	/* Dealing with geographic coordinates */
-	resample = (!Ctrl->A.active && periodic);
+	resample = ((!Ctrl->A.active || Ctrl->A.mode) && periodic);
 	/* Maximum step size (in degrees) used for interpolation of line segments along great circles (when requested) */
 	step = Ctrl->A.step / project_info.x_scale / project_info.M_PR_DEG;
 	if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
@@ -321,6 +321,7 @@ int main (int argc, char **argv)
 
 	done = FALSE;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
+	GMT_io.skip_duplicates = TRUE;	/* The inonout algorithm assumes no duplicates */
 
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -355,9 +356,10 @@ int main (int argc, char **argv)
 			lon_sum = lat_sum = 0.0;
 
 			while (!(GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE  */
+				n_read++;
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
-					exit (EXIT_FAILURE);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields,  n_expected_fields, n_read);
+					continue;
 				}
 				x[n_path] = in[GMT_X];
 				y[n_path] = in[GMT_Y];
@@ -418,13 +420,8 @@ int main (int argc, char **argv)
 			if (Ctrl->S.active) {	/* Assign 'inside' to nodes within given distance of data constrains */
 				for (k = 0; k < n_path; k++) {
 
-					if (y[k] < header.y_min || y[k] > header.y_max) continue;
-					if (periodic) {	/* Make sure we are not off by 360 somewhere */
-						x[k] -= 360.0;
-						while (x[k] < header.x_min) x[k] += 360.0;
-						if (x[k] > header.x_max) continue;
-					}
-					else if (x[k] < header.x_min || x[k] > header.x_max) continue;
+					if (GMT_y_is_outside (y[k], header.y_min, header.y_max)) continue;	/* Outside y-range */
+					if (GMT_x_is_outside (&x[k], header.x_min, header.x_max)) continue;	/* Outside x-range (or longitude) */
 
 					/* OK, this point is within bounds, but may be exactly on the border */
 
@@ -526,6 +523,7 @@ int main (int argc, char **argv)
 
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
+	GMT_io.skip_duplicates = FALSE;	/* Reset to FALSE */
 
 	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
@@ -536,17 +534,17 @@ int main (int argc, char **argv)
 	GMT_free ((void *)P.min);
 	GMT_free ((void *)P.max);
 
-	Free_Grdmask_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_grdmask_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDMASK_CTRL *C;
 	
-	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
+	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDMASK_CTRL), "New_grdmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->A.step = gmtdefs.line_step;
@@ -554,7 +552,7 @@ void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure *
 	return ((void *)C);
 }
 
-void Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C) {	/* Deallocate control structure */
+void Free_grdmask_Ctrl (struct GRDMASK_CTRL *C) {	/* Deallocate control structure */
 	if (C->G.file) free ((void *)C->G.file);	
 	GMT_free ((void *)C);	
 }
