@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvolume.c,v 1.49 2010/01/05 01:15:46 guru Exp $
+ *	$Id: grdvolume.c,v 1.52 2010/03/22 21:24:54 jluis Exp $
  *
  *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
  *	See LICENSE.TXT file for copying and redistribution conditions.
@@ -31,22 +31,22 @@
 
 struct GRDVOLUME_CTRL {
 	struct C {	/* -C */
-		BOOLEAN active;
+		GMT_LONG active;
 		double low, high, inc;
 	} C;
 	struct L {	/* -L<base> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} L;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 		char unit;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 	struct Z {	/* Z<fact>[/<shift>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double scale, offset;
 	} Z;
 };
@@ -56,7 +56,7 @@ double vol_prism_frac_y (float *z, GMT_LONG ij, GMT_LONG nx, double y0, double y
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, full = FALSE, bad, cut[4];
+	GMT_LONG error = FALSE, full = FALSE, bad, cut[4];
 
 	char *grdfile = CNULL, format[BUFSIZ];
 
@@ -72,16 +72,16 @@ int main (int argc, char **argv)
 
 	struct GRD_HEADER grd;
 
-	void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
-	void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
-	void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
-	void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
-	void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN right, double *dv, double *da);
-	void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN top, double *dv, double *da);
+	void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da);
+	void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG right, double *dv, double *da);
+	void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG top, double *dv, double *da);
 	GMT_LONG ors_find_kink (double y[], GMT_LONG n, GMT_LONG mode);
 	void *New_grdvolume_Ctrl (), Free_grdvolume_Ctrl (struct GRDVOLUME_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
 	Ctrl = (struct GRDVOLUME_CTRL *) New_grdvolume_Ctrl ();	/* Allocate and initialize a new control structure */
 	
@@ -370,15 +370,15 @@ int main (int argc, char **argv)
 
 	if (full) {
 		sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		for (c = 0; c < n_contours; c++) fprintf (GMT_stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
+		for (c = 0; c < n_contours; c++) fprintf (stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
 	}
 	else if (Ctrl->C.active) {
         	sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (GMT_stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
+		fprintf (stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
 	}
 	else {
         	sprintf (format, "%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (GMT_stdout, format, area[c], vol[c], height[c]);
+		fprintf (stdout, format, area[c], vol[c], height[c]);
 	}
 
 	GMT_free ((void *)f);
@@ -393,7 +393,7 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
+void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a SW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x1, y0, frac;
@@ -411,7 +411,7 @@ void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double
 	}
 }
 
-void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
+void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a NE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x0, y1, a, x0_1, y1_1, frac = 0.0;
@@ -434,7 +434,7 @@ void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double
 	}
 }
 
-void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
+void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a SE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x0, y1, c, x0_1, frac = 0.0;
@@ -456,7 +456,7 @@ void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double
 	}
 }
 
-void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
+void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG triangle, double *dv, double *da)
 {	/* Calculates area of a NW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x1, y0, y0_1, frac;
@@ -475,7 +475,7 @@ void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double
 	}
 }
 
-void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN right, double *dv, double *da)
+void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG right, double *dv, double *da)
 {	/* Calculates area of a NS trapezoid */
 	/* right = TRUE gets the right trapezoid, FALSE gets the left */
 	double x0, x1;
@@ -492,7 +492,7 @@ void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN right, double *d
 	}
 }
 
-void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN top, double *dv, double *da)
+void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, GMT_LONG top, double *dv, double *da)
 {	/* Calculates area of a EW trapezoid */
 	/* top = TRUE gets the top trapezoid, FALSE gets the bottom */
 	double y0, y1;
