@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
-*	$Id: mapproject.c,v 1.108 2009/09/09 23:27:03 guru Exp $
+*	$Id: mapproject.c,v 1.127 2010/10/27 22:10:19 guru Exp $
 *
-*	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+*	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
 *	See LICENSE.TXT file for copying and redistribution conditions.
 *
 *	This program is free software; you can redistribute it and/or modify
@@ -39,58 +39,58 @@
 */
 
 #include "gmt.h"
+#include "gmt_proj.h"   /* For auxiliary latitude stuff */
 
 struct MAPPROJECT_CTRL {	/* All control options for this program (except common args) */
 	/* active is TRUE if the option has been activated */
 	struct A {	/* -Ab|B|f|F<lon0>/<lat0> */
-		BOOLEAN active;
-		BOOLEAN azims;
-		BOOLEAN reverse;	/* TRUE if we want back-azimuths instead of regular azimuths */
-		BOOLEAN geodesic;	/* TRUE if we want geodesic azimuths [Default is great circle azimuths] */
+		GMT_LONG active;
+		GMT_LONG azims;
+		GMT_LONG reverse;	/* TRUE if we want back-azimuths instead of regular azimuths */
+		GMT_LONG geodesic;	/* TRUE if we want geodesic azimuths [Default is great circle azimuths] */
 		double lon, lat;	/* Fixed point of reference */
 	} A;
 	struct C {	/* -C[<false_easting>/<false_northing>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double easting, northing;	/* Shifts */
 	} C;
 	struct D {	/* -D<c|i|m|p> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char unit;
 	} D;
 	struct E {	/* -E[<datum>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_DATUM datum;	/* Contains a, f, xyz[3] */
 	} E;
 	struct F {	/* -F[k|m|n|i|c|p] */
-		BOOLEAN active;
+		GMT_LONG active;
 		char unit;
 	} F;
 	struct G {	/* -G<lon0>/<lat0>[/e|E|k|K|m|M|n|N|c|C|d|D] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;		/* 1 = distance to fixed point, 2 = cumulative distances, 3 = incremental distances, 4 = 2nd point in cols 3/4 */
-		BOOLEAN geodesic;	/* TRUE if we want geodesic distances [Default is great circle distances] */
 		double lon, lat;	/* Fixed point of reference */
 		char unit;
 	} G;
 	struct I {	/* -I */
-		BOOLEAN active;
+		GMT_LONG active;
 	} I;
 	struct L {	/* -L<line.xy>[/<unit>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;	/* 0 = dist to nearest point, 1 = also get the point, 2 = instead get seg#, pt# */
 		char *file;	/* Name of file with lines */
 		char unit;
 	} L;
 	struct Q {	/* -Q[e|d] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;	/* 1 = print =Qe, 2 print -Qd, 3 print both */
 	} Q;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 	} S;
 	struct T {	/* -T[h]<from>[/<to>] */
-		BOOLEAN active;
-		BOOLEAN heights;	/* True if we have heights */
+		GMT_LONG active;
+		GMT_LONG heights;	/* True if we have heights */
 		struct GMT_DATUM from;	/* Contains a, f, xyz[3] */
 		struct GMT_DATUM to;	/* Contains a, f, xyz[3] */
 	} T;
@@ -100,13 +100,13 @@ int main (int argc, char **argv)
 {
 	GMT_LONG k, fno, n_files = 0, x, y, n_args, unit = 0, n_slash;
 	GMT_LONG n_fields, n_expected_fields, proj_type = 0, *n_output, save[2] = {0,0}, two, n_out;
-	GMT_LONG fmt[2], pos, i, j, n = 0, n_read = 0;
+	GMT_LONG fmt[2], pos, i, j, n = 0, n_read = 0, n_read_in_seg;
 
-	BOOLEAN error = FALSE;
-	BOOLEAN nofile = TRUE, done = FALSE, first = TRUE;
-	BOOLEAN pure_ascii = FALSE, line_start = TRUE, long_verbose = FALSE;
-	BOOLEAN geodetic_calc = FALSE, greenwich = FALSE;
-	BOOLEAN datum_conv_only = FALSE, double_whammy = FALSE, shift_xy = FALSE;
+	GMT_LONG error = FALSE, do_geo_conv = FALSE;
+	GMT_LONG nofile = TRUE, done = FALSE, first = TRUE;
+	GMT_LONG pure_ascii = FALSE, line_start = TRUE, long_verbose = FALSE;
+	GMT_LONG geodetic_calc = FALSE, greenwich = FALSE;
+	GMT_LONG datum_conv_only = FALSE, double_whammy = FALSE, shift_xy = FALSE;
 
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, x_in = 0.0, y_in = 0.0, d = 0.0, s = 0.0;
 	double xmin, xmax, ymin, ymax, *in, *out, fwd_scale, inv_scale, xtmp, ytmp;
@@ -127,11 +127,11 @@ int main (int argc, char **argv)
 	PFI near_a_line;
 	PFD azimuth_func;
 
-	void *New_Mapproject_Ctrl (), Free_Mapproject_Ctrl (struct MAPPROJECT_CTRL *C);
+	void *New_mapproject_Ctrl (), Free_mapproject_Ctrl (struct MAPPROJECT_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct MAPPROJECT_CTRL *) New_Mapproject_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
+	Ctrl = (struct MAPPROJECT_CTRL *) New_mapproject_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
 	
 	out = (double *)NULL;
 
@@ -300,7 +300,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "mapproject %s - Forward and Inverse map transformations and geodesy\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: mapproject <infiles> %s %s [-C[<dx/dy>]]\n", GMT_J_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-Ab|B|f|F[<lon0/lat0>]] [-Dc|i|m|p] [-E[<datum>]] [-F[k|m|n|i|c|p]] [-G[<lon0/lat0>/][+|-][<unit>]\n");
-		fprintf (stderr, "\t[%s] [-I] [-L<line.xy>[/<unit>]] [-Q[e|d]] [-S] [-T[h]<from>[/<to>]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[%s] [-I] [-L<line.xy>[/<unit>]][+] [-Q[e|d]] [-S] [-T[h]<from>[/<to>]\n", GMT_H_OPT);
 		fprintf (stderr, "\t[-V[l]] [%s] [%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_g_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -316,7 +316,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Upper case B or F gives azimuths of geodesics using current ellipsoid.\n");
 		fprintf (stderr, "\t-C returns x/y relative to projection center [Default is relative to lower left corner]\n");
 		fprintf (stderr, "\t   Optionally append dx/dy to add (or subtract if -I) (i.e., false easting & northing) [0/0]\n");
-		fprintf (stderr, "\t   Units are plot units unless -F is set in which case the unit is meters.\n");
+		fprintf (stderr, "\t   Units are plot units unless -F[unit] is set in which case the unit is always meters.\n");
 		fprintf (stderr, "\t-D Temporarily reset MEASURE_UNIT to be c (cm), i (inch), m (meter), or p (point)\n");
 		fprintf (stderr, "\t   Cannot be used if -F is set.\n");
 		fprintf (stderr, "\t-E Convert (lon, lat, h) to Earth Centered Earth Fixed (ECEF) coordinates [-I for inverse].\n");
@@ -339,6 +339,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Unit C means Cartesian distances after first projecting the input coordinates (-R, -J).\n");
 		fprintf (stderr, "\t   Calculations uses spherical approximations.  Default unit is meters.\n");
 		fprintf (stderr, "\t   Three columns are added on output: min dist and lon, lat of the closest point on the line\n");
+		fprintf (stderr, "\t   Append + to get line segment id and fractional point number instead of lon/lat.\n");
 		fprintf (stderr, "\t-Q list projection parameters and exit.  For subsets [Default is all] use\n");
 		fprintf (stderr, "\t   -Qe shows ellipsoid parameters\n");
 		fprintf (stderr, "\t   -Qd shows datum parameters\n");
@@ -559,6 +560,15 @@ int main (int argc, char **argv)
 				}
 			}
 		}
+		else if (GMT_io.in_col_type[0] & GMT_IS_GEO && proj_type == 0 && !GMT_IS_SPHERICAL) {
+			do_geo_conv = TRUE;
+			/* Will need spherical trig so convert to geocentric latitudes */
+			for (i = 0; i < xyline->n_segments; i++) {
+				for (j = 0; j < xyline->segment[i]->n_rows; j++) {
+					xyline->segment[i]->coord[GMT_Y][j] = GMT_lat_swap (xyline->segment[i]->coord[GMT_Y][j], GMT_LATSWAP_G2O);	/* Convert to geocentric */
+				}
+			}
+		}
 	}
 
 	/* Now we are ready to take on some input values */
@@ -574,7 +584,7 @@ int main (int argc, char **argv)
 		GMT_io.in_col_type[2] = GMT_io.out_col_type[2] = GMT_IS_FLOAT;
 	}
 
-	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : GMT_MAX_COLUMNS;
 	if (GMT_io.ncol[GMT_OUT])	/* Want binary output to be limitied to first ncol[1] columns */
 		n_output = &GMT_io.ncol[GMT_OUT];
 	else	/* Default to the number of input fields */
@@ -592,8 +602,14 @@ int main (int argc, char **argv)
 	two = (Ctrl->E.active || (Ctrl->T.active && GMT_datum.h_given)) ? 3 : 2;	/* # of output points from conversion */
 
 	if (shift_xy && Ctrl->F.active) {	/* Use same units in -C and -F */
-		Ctrl->C.easting *= u_scale;
-		Ctrl->C.northing *= u_scale;
+		if (Ctrl->I.active) {
+			Ctrl->C.easting /= u_scale;
+			Ctrl->C.northing /= u_scale;
+		}
+		else {
+			Ctrl->C.easting *= u_scale;
+			Ctrl->C.northing *= u_scale;
+		}
 	}
 	if (Ctrl->G.mode >= 2 && proj_type == 2) {	/* Must project the fixed point here */
 		GMT_geo_to_xy (Ctrl->G.lon, Ctrl->G.lat, &xtmp, &ytmp);
@@ -628,7 +644,7 @@ int main (int argc, char **argv)
 		fmt[1] = 1;
 	}
 
-	n = 0;
+	n = n_read_in_seg = 0;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
 
@@ -650,7 +666,7 @@ int main (int argc, char **argv)
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
 				not_used = GMT_fgets (line, BUFSIZ, fp);
-				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", line);
+				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) GMT_fputs (line, GMT_stdout);
 			}
 			first = FALSE;
 		}
@@ -664,15 +680,20 @@ int main (int argc, char **argv)
 				while (GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {
 					GMT_write_segmentheader (GMT_stdout, n_expected_fields);
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
+					n_read_in_seg = 0;
 				}
 				if (GMT_REC_IS_GAP) {
 					GMT_write_segmentheader (GMT_stdout, *n_output);
 					GMT_io.status = 0;	/* Done with gap */
 					line_start = TRUE;
+					n_read_in_seg = 0;
 				}
 				if (GMT_REC_IS_EOF) continue;
 
 				while (!GMT_REC_IS_LINE_BREAK && !GMT_REC_IS_EOF) {	/* Keep going until FALSE or = 2 segment header */
+
+					n_read++;
+					n_read_in_seg++;
 					if (GMT_REC_IS_ERROR && n_fields < 2) {
 						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 						continue;
@@ -714,9 +735,10 @@ int main (int argc, char **argv)
 						in[GMT_Y] = out[GMT_Y];
 						GMT_conv_datum (in, out);
 					}
-
-					n_read++;
-					if (Ctrl->S.active && GMT_map_outside (out[GMT_X], out[GMT_Y])) continue;
+					if (Ctrl->S.active && GMT_map_outside (out[GMT_X], out[GMT_Y])) {
+						n_fields = GMT_input (fp, &n_expected_fields, &in);
+						continue;
+					}
 					if (gmtdefs.verbose) {
 						x_in_min = MIN (x_in_min, x_in);
 						x_in_max = MAX (x_in_max, x_in);
@@ -742,15 +764,15 @@ int main (int argc, char **argv)
 						GMT_strtok (line, " \t,", &pos, p);	/* Returns xstring and update pos */
 						GMT_strtok (line, " \t,", &pos, p);	/* Returns ystring and update pos */
 						GMT_ascii_output_one (GMT_stdout, out[x], 0);
-						fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+						GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 						GMT_ascii_output_one (GMT_stdout, out[y], 1);
-						fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+						GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 						if (Ctrl->E.active) {
 							GMT_strtok (line, " \t,", &pos, p);	/* Returns zstring and update pos */
 							GMT_ascii_output_one (GMT_stdout, out[2], 2);
-							fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+							GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 						}
-						fprintf (GMT_stdout, "%s\n", &line[pos]);	/* Start of user text */
+						GMT_fputs (&line[pos], GMT_stdout);	GMT_fputs("\n", GMT_stdout);	/* Start of user text */
 					}
 					else {	/* Simply copy other columns and output */
 						for (k = two; k < *n_output; k++) out[k] = in[k];
@@ -771,11 +793,13 @@ int main (int argc, char **argv)
 					GMT_write_segmentheader (GMT_stdout, n_expected_fields);
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
 					line_start = TRUE;
+					n_read_in_seg = 0;
 				}
 				if (GMT_REC_IS_GAP) {
 					GMT_write_segmentheader (GMT_stdout, *n_output);
 					GMT_io.status = 0;	/* Done with gap */
 					line_start = TRUE;
+					n_read_in_seg = 0;
 				}
 				if (GMT_REC_IS_EOF) continue;
 
@@ -786,10 +810,14 @@ int main (int argc, char **argv)
 					}
 					if (!out) out = (double *) GMT_memory (VNULL, (size_t)n_expected_fields+3, sizeof (double), GMT_program);
 					n_read++;
+					n_read_in_seg++;
 
 					/* Because -: is processed in GMT_input we use [0] for lon and [1] for y always */
 
-					if (Ctrl->S.active && GMT_map_outside (in[GMT_X], in[GMT_Y])) continue;
+					if (Ctrl->S.active && GMT_map_outside (in[GMT_X], in[GMT_Y])) {
+						n_fields = GMT_input (fp, &n_expected_fields, &in);
+						continue;
+					}
 
 					if (datum_conv_only) {
 						GMT_conv_datum (in, out);
@@ -870,12 +898,15 @@ int main (int argc, char **argv)
 						else if (Ctrl->L.active) {	/* Compute closest distance to line */
 							if (proj_type == 2)	/* Using projected coordinates */
 								(void) near_a_line (out[GMT_X], out[GMT_Y], xyline, Ctrl->L.mode, &d, &xnear, &ynear);
-							else			/* Using input coordinates */
-								(void) near_a_line (in[GMT_X], in[GMT_Y], xyline, Ctrl->L.mode, &d, &xnear, &ynear);
+							else {			/* Using input coordinates */
+								y_in = (do_geo_conv) ? GMT_lat_swap (in[GMT_Y], GMT_LATSWAP_G2O) : in[GMT_Y];				/* Convert to geocentric */
+								(void) near_a_line (in[GMT_X], y_in, xyline, Ctrl->L.mode, &d, &xnear, &ynear);
+								if (do_geo_conv && Ctrl->L.mode != 3) ynear = GMT_lat_swap (ynear, GMT_LATSWAP_O2G);		/* Convert back to geodetic */
+							}
 							d *= d_scale;
 						}
 						else if (Ctrl->A.azims) {	/* Azimuth from previous point */
-							if (n_read == 1) {	/* First point has undefined azimuth since there is no previous point */
+							if (n_read_in_seg == 1) {	/* First point has undefined azimuth since there is no previous point */
 								d = GMT_d_NaN;
 							}
 							else {
@@ -902,19 +933,19 @@ int main (int argc, char **argv)
 							GMT_strtok (line, " \t,", &pos, p);	/* Returns ystring */
 
 							GMT_ascii_output_one (GMT_stdout, in[x], 0);
-							fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+							GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 							GMT_ascii_output_one (GMT_stdout, in[y], 1);
-							fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
-							fprintf (GMT_stdout, "%s", &line[pos]);
-							fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+							GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
+							GMT_fputs (&line[pos], GMT_stdout);
+							GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 							GMT_ascii_output_one (GMT_stdout, d, 2);
 							if (Ctrl->L.active) {
-								fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+								GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 								GMT_ascii_output_one (GMT_stdout, xnear, fmt[0]);
-								fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+								GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 								GMT_ascii_output_one (GMT_stdout, ynear, fmt[1]);
 							}
-							fprintf (GMT_stdout, "\n");
+							GMT_fputs ("\n", GMT_stdout);
 						}
 						else {	/* Simply copy other columns and output */
 							for (k = 0; k < *n_output; k++) out[k] = in[k];
@@ -948,15 +979,15 @@ int main (int argc, char **argv)
 							GMT_strtok (line, " \t,", &pos, p);	/* Returns xstring and update pos */
 							GMT_strtok (line, " \t,", &pos, p);	/* Returns ystring and update pos */
 							GMT_ascii_output_one (GMT_stdout, out[x], 0);
-							fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+							GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 							GMT_ascii_output_one (GMT_stdout, out[y], 1);
-							fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+							GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 							if (Ctrl->E.active || (Ctrl->T.active && GMT_datum.h_given)) {
 								GMT_strtok (line, " \t,", &pos, p);	/* Returns zstring and update pos */
 								GMT_ascii_output_one (GMT_stdout, out[2], 2);
-								fprintf (GMT_stdout, "%s", gmtdefs.field_delimiter);
+								GMT_fputs (gmtdefs.field_delimiter, GMT_stdout);
 							}
-							fprintf (GMT_stdout, "%s\n", &line[pos]);
+							GMT_fputs (&line[pos], GMT_stdout);	GMT_fputs("\n", GMT_stdout);
 						}
 						else {	/* Simply copy other columns and output */
 							for (k = two; k < *n_output; k++) out[k] = in[k];
@@ -1003,17 +1034,17 @@ int main (int argc, char **argv)
 
 	if (Ctrl->L.active) GMT_free_table (xyline);
 
-	Free_Mapproject_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_mapproject_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Mapproject_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_mapproject_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct MAPPROJECT_CTRL *C;
 	
-	C = (struct MAPPROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct MAPPROJECT_CTRL), "New_Mapproject_Ctrl");
+	C = (struct MAPPROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct MAPPROJECT_CTRL), "New_mapproject_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
@@ -1022,7 +1053,7 @@ void *New_Mapproject_Ctrl () {	/* Allocate and initialize a new control structur
 	return ((void *)C);
 }
 
-void Free_Mapproject_Ctrl (struct MAPPROJECT_CTRL *C) {	/* Deallocate control structure */
+void Free_mapproject_Ctrl (struct MAPPROJECT_CTRL *C) {	/* Deallocate control structure */
 	if (C->L.file) free ((void *)C->L.file);	
 	GMT_free ((void *)C);	
 }
