@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psbasemap.c,v 1.48 2009/05/13 21:06:43 guru Exp $
+ *	$Id: psbasemap.c,v 1.54 2010/10/13 23:24:30 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -30,23 +30,23 @@
 
 struct PSBASEMAP_CTRL {
 	struct E {	/* -Eazim/elev */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth, elevation;
 	} E;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct L {	/* -L */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_MAP_SCALE item;
 	} L;
 	struct T {	/* -L */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_MAP_ROSE item;
 	} T;
 	struct Z {	/* -Z<z_level> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double level;
 	} Z;
 };
@@ -55,89 +55,87 @@ int main (int argc, char **argv)
 {
 	GMT_LONG i;
 
-	BOOLEAN error = FALSE;
+	GMT_LONG error = FALSE;
 
 	double w = 0.0, e = 0.0, s = 0.0, n = 0.0;
 
 	struct PSBASEMAP_CTRL *Ctrl;
 
-	void *New_Psbasemap_Ctrl (), Free_Psbasemap_Ctrl (struct PSBASEMAP_CTRL *C);
+	void *New_psbasemap_Ctrl (), Free_psbasemap_Ctrl (struct PSBASEMAP_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PSBASEMAP_CTRL *)New_Psbasemap_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct PSBASEMAP_CTRL *)New_psbasemap_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	for (i = 1; i < argc; i++) {
-		if (argv[i][0] != '-') {
-			error = TRUE;
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Options must start with leading -. Your argument \"%s\" is unrecognized\n", GMT_program, argv[i]);
-			continue;
-		}
+		if (argv[i][0] == '-') {
+			switch (argv[i][1]) {
 
-		switch (argv[i][1]) {
+				/* Common parameters */
 
-			/* Common parameters */
+				case 'B':
+				case 'J':
+				case 'K':
+				case 'O':
+				case 'P':
+				case 'R':
+				case 'U':
+				case 'V':
+				case 'X':
+				case 'x':
+				case 'Y':
+				case 'y':
+				case 'c':
+				case '\0':
+					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
+					break;
 
-			case 'B':
-			case 'J':
-			case 'K':
-			case 'O':
-			case 'P':
-			case 'R':
-			case 'U':
-			case 'V':
-			case 'X':
-			case 'x':
-			case 'Y':
-			case 'y':
-			case 'c':
-			case '\0':
-				error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
-				break;
+				/* Supplemental options */
 
-			/* Supplemental options */
+				case 'E':
+					Ctrl->E.active = TRUE;
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					break;
 
-			case 'E':
-				Ctrl->E.active = TRUE;
-				error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
-				break;
+				case 'G':
+					Ctrl->G.active = TRUE;
+					if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
+						GMT_fill_syntax ('G', " ");
+						error++;
+					}
+					break;
 
-			case 'G':
-				Ctrl->G.active = TRUE;
-				if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
-					GMT_fill_syntax ('G', " ");
-					error++;
-				}
-				break;
+				case 'L':
+					Ctrl->L.active = TRUE;
+					error += GMT_getscale (&argv[i][2], &Ctrl->L.item);
+					break;
 
-			case 'L':
-				Ctrl->L.active = TRUE;
-				error += GMT_getscale (&argv[i][2], &Ctrl->L.item);
-				break;
+				case 'T':
+					Ctrl->T.active = TRUE;
+					error += GMT_getrose (&argv[i][2], &Ctrl->T.item);
+					break;
 
-			case 'T':
-				Ctrl->T.active = TRUE;
-				error += GMT_getrose (&argv[i][2], &Ctrl->T.item);
-				break;
+				case 'Z':
+					if (argv[i][2]) {
+						Ctrl->Z.level = atof (&argv[i][2]);
+						Ctrl->Z.active = TRUE;
+					}
+					else {
+						error++;
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -Z:  Must append a z-value\n", GMT_program);
+					}
+					break;
 
-			case 'Z':
-				if (argv[i][2]) {
-					Ctrl->Z.level = atof (&argv[i][2]);
-					Ctrl->Z.active = TRUE;
-				}
-				else {
-					error++;
-					fprintf (stderr, "%s: GMT SYNTAX ERROR -Z:  Must append a z-value\n", GMT_program);
-				}
-				break;
+				/* Illegal options */
 
-			/* Illegal options */
-
-			default:
-				error = TRUE;
-				GMT_default_error (argv[i][1]);
-				break;
+				default:
+					error = TRUE;
+					GMT_default_error (argv[i][1]);
+					break;
+			}
 		}
+		else
+			fprintf (stderr, "%s: Warning: Ignoring filename %s\n", GMT_program, argv[i]);
 	}
 
 	if (GMT_give_synopsis_and_exit || argc == 1) {
@@ -202,7 +200,7 @@ int main (int argc, char **argv)
 	if (Ctrl->G.active) {
 		double *x, *y;
 		GMT_LONG np;
-		BOOLEAN donut;
+		GMT_LONG donut;
 		np = GMT_map_clip_path (&x, &y, &donut);
 		GMT_fill (x, y, (1 + donut) * np, &Ctrl->G.fill, FALSE);
 		GMT_free ((void *)x);
@@ -221,17 +219,17 @@ int main (int argc, char **argv)
 	
 	GMT_plotend ();
 
-	Free_Psbasemap_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_psbasemap_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void *New_Psbasemap_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_psbasemap_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSBASEMAP_CTRL *C;
 	
-	C = (struct PSBASEMAP_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSBASEMAP_CTRL), "New_Psbasemap_Ctrl");
+	C = (struct PSBASEMAP_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSBASEMAP_CTRL), "New_psbasemap_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
@@ -243,6 +241,6 @@ void *New_Psbasemap_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void Free_Psbasemap_Ctrl (struct PSBASEMAP_CTRL *C) {	/* Deallocate control structure */
+void Free_psbasemap_Ctrl (struct PSBASEMAP_CTRL *C) {	/* Deallocate control structure */
 	GMT_free ((void *)C);	
 }
