@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psmask.c,v 1.79 2009/07/11 03:16:31 guru Exp $
+ *	$Id: psmask.c,v 1.92 2010/10/27 22:31:21 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -42,55 +42,53 @@
 
 struct PSMASK_CTRL {
 	struct C {	/* -C */
-		BOOLEAN active;
+		GMT_LONG active;
 	} C;
 	struct D {	/* -D<dumpfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 		GMT_LONG min_pts;
 	} D;
 	struct E {	/* -Eazim/elev */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth, elevation;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -Idx[/dy] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xinc, yinc;
 	} I;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double radius;
 		char unit;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 };
 
 struct PSMASK_INFO {
-	BOOLEAN first_dump;
+	GMT_LONG first_dump;
 	GMT_LONG p[5], i_off[5], j_off[5], k_off[5], offset;
 	unsigned int bit[32];
 };
 
 int main (int argc, char **argv)
 {
-	GMT_LONG ij, n, nm;
+	GMT_LONG ij, n, nm, n_read, n_alloc;
 	GMT_LONG i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
 	GMT_LONG section, n_fields, distance_flag = 0, n_plus = 0, k;
-	GMT_LONG n_read, n_alloc;
-	
-	BOOLEAN error = FALSE, first = TRUE, node_only;
+	GMT_LONG error = FALSE, first = TRUE, node_only;
 
 	char line[BUFSIZ], *grd, *not_used = NULL;
 
@@ -104,17 +102,17 @@ int main (int argc, char **argv)
 	struct PSMASK_CTRL *Ctrl;
 
 #ifdef DEBUG
-	BOOLEAN debug = FALSE;
+	GMT_LONG debug = FALSE;
 #endif
 	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], GMT_LONG id, GMT_LONG flag);
 	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, GMT_LONG id, char *file);
 	void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e);
 	GMT_LONG clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, GMT_LONG *edge, GMT_LONG first, double **x, double **y, GMT_LONG *max);
-	void *New_Psmask_Ctrl (), Free_Psmask_Ctrl (struct PSMASK_CTRL *C);
+	void *New_psmask_Ctrl (), Free_psmask_Ctrl (struct PSMASK_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PSMASK_CTRL *)New_Psmask_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct PSMASK_CTRL *)New_psmask_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	memset ((void *)&info, 0, sizeof (struct PSMASK_INFO));
 	info.first_dump = TRUE;
@@ -294,7 +292,11 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Must also specify a tile color with -G\n", GMT_program);
 			error++;
 		}
-		if (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0) {
+		if (!Ctrl->I.active) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -I option\n", GMT_program);
+			error++;
+		}
+		else if (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  Must specify positive increments\n", GMT_program);
 			error++;
 		}
@@ -311,7 +313,7 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 			error++;
 		}
-		h.node_offset = Ctrl->F.active;
+		h.node_offset = (int)Ctrl->F.active;
 		h.x_inc = Ctrl->I.xinc;
 		h.y_inc = Ctrl->I.yinc;
 		h.xy_off = 0.5 * h.node_offset;
@@ -324,7 +326,7 @@ int main (int argc, char **argv)
 
 	/* Now test also if we must provide a default -J */
 	if (Ctrl->D.active && project_info.projection < 0) {		/* Is this the right way of testing it? */
-		GMT_parse_J_option ("x1");
+		GMT_parse_J_option ("x1d");
 	}
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
@@ -333,6 +335,15 @@ int main (int argc, char **argv)
 	}
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
+	if (distance_flag) {
+		shrink = cosd (0.5 * (h.y_min + h.y_max));
+		di = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink) + GMT_CONV_LIMIT);
+		dj = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc) + GMT_CONV_LIMIT);
+	}
+	else {
+		di = irint (0.5 * Ctrl->S.radius / h.x_inc + GMT_CONV_LIMIT);
+		dj = irint (0.5 * Ctrl->S.radius / h.y_inc + GMT_CONV_LIMIT);
+	}
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	if (!Ctrl->C.active) GMT_err_fail (GMT_map_setup (h.x_min, h.x_max, h.y_min, h.y_max), "");
@@ -340,6 +351,11 @@ int main (int argc, char **argv)
 	if (!project_info.x_off_supplied && GMT_ps.overlay) GMT_ps.x_origin = 0.0;
 	if (!project_info.y_off_supplied && GMT_ps.overlay) GMT_ps.y_origin = 0.0;
 
+	if (Ctrl->C.active)
+		GMT_ps.clip = -1;	/* Signal that this program terminates clipping that initiated prior to this process */
+	else if (!Ctrl->T.active)
+		GMT_ps.clip = +1;	/* Signal that this program initiates clipping that wil outlive this process */
+		
 	GMT_plotinit (argc, argv);
 
 	if (!Ctrl->C.active) {	/* Start new clip_path */
@@ -360,15 +376,6 @@ int main (int argc, char **argv)
 
 		/* Add GMT_CONV_LIMIT to ensure that special case radius = inc --> irint(0.5) actually rounds to 1 */
 		
-		if (distance_flag) {
-			shrink = cosd (0.5 * (h.y_min + h.y_max));
-			di = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink) + GMT_CONV_LIMIT);
-			dj = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc) + GMT_CONV_LIMIT);
-		}
-		else {
-			di = irint (0.5 * Ctrl->S.radius / h.x_inc + GMT_CONV_LIMIT);
-			dj = irint (0.5 * Ctrl->S.radius / h.y_inc + GMT_CONV_LIMIT);
-		}
 		if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
 		switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
 
@@ -407,12 +414,16 @@ int main (int argc, char **argv)
 			}
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
-				exit (EXIT_FAILURE);
-			}
 			n_read++;
 
+			if (GMT_io.status & GMT_IO_MISMATCH) {
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				continue;
+			}
+
+			if (GMT_y_is_outside (in[GMT_Y], h.y_min, h.y_max)) continue;		/* Outside y-range */
+			if (GMT_x_is_outside (&in[GMT_X], h.x_min, h.x_max)) continue;	/* Outside x-range (or longitude) */
+
 			/* Determine the node closest to the data point */
 
 			i = GMT_x_to_i (in[GMT_X], h.x_min, h.x_inc, h.xy_off, h.nx);
@@ -517,21 +528,20 @@ int main (int argc, char **argv)
 					
 					if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
 
-					/* First truncate agains left border */
+						/* First truncate against left border */
 
-						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
+						GMT_n_plot = GMT_truncate (xp, yp, plot_n, start, -1);
 						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &Ctrl->G.fill, FALSE);
 
-						/* Then truncate agains right border */
+						/* Then truncate against right border */
 
-						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
+						GMT_n_plot = GMT_truncate (xp, yp, plot_n, start, +1);
 						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &Ctrl->G.fill, FALSE);
 					}
-					else {
+					else
 						GMT_fill (xp, yp, plot_n, &Ctrl->G.fill, FALSE);
-					}
 					GMT_free ((void *)xp);
 					GMT_free ((void *)yp);
 				}
@@ -557,7 +567,7 @@ int main (int argc, char **argv)
 
 	GMT_plotend ();
 
-	Free_Psmask_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_psmask_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
@@ -641,7 +651,7 @@ GMT_LONG clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *
 	static GMT_LONG i0, j0, side;
 	GMT_LONG ij;
 	GMT_LONG i, j, n = 0, n_edges, edge_word, edge_bit;
-	BOOLEAN go_on = TRUE;
+	GMT_LONG go_on = TRUE;
 	GMT_LONG trace_clip_contours (struct PSMASK_INFO *info, char *grd, GMT_LONG *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, GMT_LONG i, GMT_LONG j, GMT_LONG kk, GMT_LONG *max);
 	 
 	 
@@ -850,10 +860,10 @@ void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e)
 	}
 }
 
-void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSMASK_CTRL *C;
 	
-	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSMASK_CTRL), "New_Psmask_Ctrl");
+	C = (struct PSMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSMASK_CTRL), "New_psmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -863,7 +873,7 @@ void *New_Psmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	return ((void *)C);
 }
 
-void Free_Psmask_Ctrl (struct PSMASK_CTRL *C) {	/* Deallocate control structure */
+void Free_psmask_Ctrl (struct PSMASK_CTRL *C) {	/* Deallocate control structure */
 	if (C->D.file) free ((void *)C->D.file);	
 	GMT_free ((void *)C);	
 }
