@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psrose.c,v 1.75 2009/07/11 03:16:31 guru Exp $
+ *	$Id: psrose.c,v 1.83 2010/05/10 23:12:32 remko Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -37,74 +37,74 @@
 struct PSROSE_CTRL {	/* All control options for this program (except common args) */
 	/* active is TRUE if the option has been activated */
 	struct A {	/* -A<sector_angle>[r] */
-		BOOLEAN active;
-		BOOLEAN rose;
+		GMT_LONG active;
+		GMT_LONG rose;
 		double inc;
 	} A;
 	struct C {	/* -C[<modefile>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} C;
 	struct D {	/* -D */
-		BOOLEAN active;
+		GMT_LONG active;
 	} D;
 	struct E {	/* -Eazimuth/elevation */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth, elevation;
 	} E;
 	struct F {	/* -F */
-		BOOLEAN active;
+		GMT_LONG active;
 	} F;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -I */
-		BOOLEAN active;
+		GMT_LONG active;
 	} I;
 	struct L {	/* -L */
-		BOOLEAN active;
+		GMT_LONG active;
 		char w[GMT_LONG_TEXT];
 		char e[GMT_LONG_TEXT];
 		char s[GMT_LONG_TEXT];
 		char n[GMT_LONG_TEXT];
 	} L;
 	struct M {	/* -M<params> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double width, length, thickness;
 		int rgb[3];
 	} M;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct S {	/* -Sscale[n] */
-		BOOLEAN active;
-		BOOLEAN normalize;
+		GMT_LONG active;
+		GMT_LONG normalize;
 		double scale;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 	struct W {	/* -W<pen> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_PEN pen;
 	} W;
 	struct Z {	/* -Zscale */
-		BOOLEAN active;
+		GMT_LONG active;
 		double scale;
 	} Z;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, find_mean = FALSE, half_only = FALSE, windrose = TRUE;
-	BOOLEAN automatic = FALSE, sector_plot = FALSE;
+	GMT_LONG error = FALSE, find_mean = FALSE, half_only = FALSE, windrose = TRUE;
+	GMT_LONG automatic = FALSE, sector_plot = FALSE;
 
 	char text[BUFSIZ], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"}, *not_used = NULL;
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT];
 
 	GMT_LONG n_bins, n_annot;
-	GMT_LONG status, n_alpha, n_modes, n_fields, n_expected_fields;
+	GMT_LONG status, n_alpha, n_modes, n_read, n_fields, n_expected_fields;
 	GMT_LONG i, bin, n = 0, n_alloc = GMT_CHUNK;
 
 	double max = 0.0, radius, az, x_origin, y_origin, tmp, one_or_two = 1.0, s, c, half_bin_width;
@@ -120,11 +120,11 @@ int main (int argc, char **argv)
 
 	void plot3d (double x, double y, GMT_LONG pen);
 	void arc3d (double x, double y, double r, double a1, double a2, GMT_LONG status);
-	void *New_Psrose_Ctrl (), Free_Psrose_Ctrl (struct PSROSE_CTRL *C);
+	void *New_psrose_Ctrl (), Free_psrose_Ctrl (struct PSROSE_CTRL *C);
 
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PSROSE_CTRL *) New_Psrose_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct PSROSE_CTRL *) New_psrose_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	asize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 	lsize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
@@ -395,16 +395,18 @@ int main (int argc, char **argv)
 
 	/* Read data and do some stats */
 
-	n = 0;
+	n = n_read = 0;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 
 	n_fields = GMT_input (fp,  &n_expected_fields, &in);
 
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+		n_read++;
+
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n);
-			exit (EXIT_FAILURE);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
+			continue;
 		}
 
 		length[n] = in[GMT_X];
@@ -503,11 +505,11 @@ int main (int argc, char **argv)
 
 	if (frame_info.plot && GMT_IS_ZERO (frame_info.axis[1].item[GMT_ANNOT_UPPER].interval)) frame_info.axis[1].item[GMT_ANNOT_UPPER].interval = frame_info.axis[1].item[GMT_GRID_UPPER].interval = 30.0;
 
-	/* Ready to plot */
+	/* Ready to plot.  So set up GMT projections (not used by psrose), we set region to actual plot width and scale to 1 */
 
 	project_info.projection = GMT_LINEAR;
 	project_info.region = TRUE;
-	project_info.pars[0] = project_info.pars[1] = (Ctrl->S.normalize) ? 1.0 : Ctrl->S.scale / max_radius;
+	project_info.pars[0] = project_info.pars[1] = 1.0;
 	GMT_err_fail (GMT_map_setup (-Ctrl->S.scale, Ctrl->S.scale, -Ctrl->S.scale, Ctrl->S.scale), "");
 
 	GMT_plotinit (argc, argv);
@@ -715,7 +717,7 @@ int main (int argc, char **argv)
 	GMT_free ((void *)mode_length);
 	GMT_free ((void *)mode_direction);
 
-	Free_Psrose_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_psrose_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
@@ -741,13 +743,13 @@ void arc3d (double x, double y, double r, double a1, double a2, GMT_LONG status)
 void plot3d (double x, double y, GMT_LONG pen)
 {
 	if (project_info.three_D) GMT_xyz_to_xy (x, y, project_info.z_level, &x, &y);
-	ps_plot (x, y, pen);
+	ps_plot (x, y, (int)pen);
 }
 
-void *New_Psrose_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_psrose_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSROSE_CTRL *C;
 	
-	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSROSE_CTRL), "New_Psrose_Ctrl");
+	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSROSE_CTRL), "New_psrose_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
@@ -769,7 +771,7 @@ void *New_Psrose_Ctrl () {	/* Allocate and initialize a new control structure */
 	return ((void *)C);
 }
 
-void Free_Psrose_Ctrl (struct PSROSE_CTRL *C) {	/* Deallocate control structure */
+void Free_psrose_Ctrl (struct PSROSE_CTRL *C) {	/* Deallocate control structure */
 	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
