@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: psscale.c,v 1.104 2010/01/14 07:15:19 guru Exp $
+ *	$Id: psscale.c,v 1.109 2010/10/13 23:24:30 guru Exp $
  *
  *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
  *	See LICENSE.TXT file for copying and redistribution conditions.
@@ -43,54 +43,54 @@
 
 struct PSSCALE_CTRL {
 	struct A {	/* -A */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;
 	} A;
 	struct C {	/* -C<cptfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} C;
 	struct D {	/* -D<xpos/ypos/length/width[h]> */
-		BOOLEAN active;
-		BOOLEAN horizontal;
+		GMT_LONG active;
+		GMT_LONG horizontal;
 		double x, y, width, length;
 	} D;
 	struct E {	/* -E[b|f][<length>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;
 		double length;
 	} E;
 	struct I {	/* -I[<intens>|<min_i>/<max_i>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		double min, max;
 	} I;
 	struct M {	/* -M */
-		BOOLEAN active;
+		GMT_LONG active;
 	} M;
 	struct N {	/* -N<dpi> */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG dpi;
 	} N;
 	struct L {	/* -L[i][<gap>] */
-		BOOLEAN active;
-		BOOLEAN interval;
+		GMT_LONG active;
+		GMT_LONG interval;
 		double spacing;
 	} L;
 	struct Q {	/* -Q */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Q;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 	} S;
 	struct Z {	/* -Z<zfile> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} Z;
 };
 
 int main(int argc, char **argv)
 {
-	BOOLEAN error = FALSE, save_unix_time, must_shift_back = FALSE, B_set = FALSE;
+	GMT_LONG error = FALSE, save_unix_time, must_shift_back = FALSE, B_set = FALSE;
 
 	char flag, line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
 	char txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT], text[GMT_LONG_TEXT];
@@ -104,12 +104,12 @@ int main(int argc, char **argv)
 
 	struct PSSCALE_CTRL *Ctrl;
 
-	void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG bit_dpi, GMT_LONG flip, BOOLEAN B_set, BOOLEAN equi, 
-	   BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, GMT_LONG extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome);
+	void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG bit_dpi, GMT_LONG flip, GMT_LONG B_set, GMT_LONG equi, 
+	   GMT_LONG horizontal, GMT_LONG logscl, GMT_LONG intens, double *max_intens, GMT_LONG skip_lines, GMT_LONG extend, double e_length, double gap, GMT_LONG interval_annot, GMT_LONG monochrome);
 
 	void *New_psscale_Ctrl (), Free_psscale_Ctrl (struct PSSCALE_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
 	Ctrl = (struct PSSCALE_CTRL *)New_psscale_Ctrl ();	/* Allocate and initialize a new control structure */
 
@@ -160,13 +160,14 @@ int main(int argc, char **argv)
 					flag = argv[i][n];
 					if (flag == 'h' || flag == 'H') {
 						Ctrl->D.horizontal = TRUE;
-						argv[i][n] = 0;
+						argv[i][n] = 0;	/* Temporarily remove the flag */
 					}
 					sscanf (&argv[i][2], "%[^/]/%[^/]/%[^/]/%s", txt_a, txt_b, txt_c, txt_d);
 					Ctrl->D.x   = GMT_convert_units (txt_a, GMT_INCH);
 					Ctrl->D.y   = GMT_convert_units (txt_b, GMT_INCH);
 					Ctrl->D.length = GMT_convert_units (txt_c, GMT_INCH);
 					Ctrl->D.width  = GMT_convert_units (txt_d, GMT_INCH);
+					if (Ctrl->D.horizontal) argv[i][n] = flag;	/* Restore the flag */
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
@@ -229,6 +230,8 @@ int main(int argc, char **argv)
 					break;
 			}
 		}
+		else
+			fprintf (stderr, "%s: Warning: Ignoring filename %s\n", GMT_program, argv[i]);
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
@@ -453,12 +456,12 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG bit_dpi, GMT_LONG flip, BOOLEAN B_set, BOOLEAN equi, BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, GMT_LONG extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome)
+void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG bit_dpi, GMT_LONG flip, GMT_LONG B_set, GMT_LONG equi, GMT_LONG horizontal, GMT_LONG logscl, GMT_LONG intens, double *max_intens, GMT_LONG skip_lines, GMT_LONG extend, double e_length, double gap, GMT_LONG interval_annot, GMT_LONG monochrome)
 {
 	GMT_LONG i, ii, id, j, nb, ndec = -1, dec, p_val, depth;
 	GMT_LONG nx = 0, ny = 0, nm, barmem, k, justify, l_justify, this_just, use_labels = 0, Label_justify;
 	int rgb[3], rrggbb[3];
-	BOOLEAN reverse, all = TRUE, use_image, center = FALSE, const_width = TRUE, do_annot;
+	GMT_LONG reverse, all = TRUE, use_image, center = FALSE, const_width = TRUE, do_annot;
 	char format[GMT_LONG_TEXT], text[GMT_LONG_TEXT], test[GMT_LONG_TEXT], unit[GMT_LONG_TEXT], label[GMT_LONG_TEXT];
 	unsigned char *bar = 0, *tmp = 0;
 	double off, annot_off, label_off, len, len2, size, x0, x1, dx, xx, dir, y_base, y_annot, y_label;
@@ -859,7 +862,7 @@ void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG b
 			memcpy ((void *)&frame_info.axis[1], (void *)&frame_info.axis[0], sizeof (struct GMT_PLOT_AXIS));
 			d_swap (project_info.x_scale, project_info.y_scale);
 			d_swap (project_info.x0, project_info.y0);
-			i_swap (project_info.xyz_projection[0], project_info.xyz_projection[1]);
+			l_swap (project_info.xyz_projection[0], project_info.xyz_projection[1]);
 			tmp = GMT_x_forward; GMT_y_forward = GMT_x_forward; GMT_x_forward = tmp;
 			for (i = 0; i < 5; i++) frame_info.axis[1].item[i].parent = 1;
 			GMT_xy_axis (-y_base, 0.0, length, start_val, stop_val, &frame_info.axis[1], (flip & 1), frame_info.side[0]-1);
