@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: pswiggle.c,v 1.65 2009/07/11 03:16:31 guru Exp $
+ *	$Id: pswiggle.c,v 1.73 2010/05/19 23:40:15 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -36,48 +36,49 @@
 
 struct PSWIGGLE_CTRL {
 	struct A {	/* -A<azimuth> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} A;
 	struct C {	/* -C<center> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} C;
-	struct D {	/* -D<gap> */
-		BOOLEAN active;
+	struct D {	/* -D[x]<gap> */
+		GMT_LONG active;
+		GMT_LONG mode;	/* 0 is km, 1 is projected inches */
 		double value;
 	} D;
 	struct E {	/* -Eazim/elev */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth, elevation;
 	} E;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -I<azimuth> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} I;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct S {	/* -S[x]<lon0>/<lat0>/<length>/<units> */
-		BOOLEAN active;
-		BOOLEAN cartesian;
+		GMT_LONG active;
+		GMT_LONG cartesian;
 		double lon, lat, length;
 		char *label;
 	} S;
 	struct T {	/* -T<pen> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_PEN pen;
 	} T;
 	struct W {	/* -W<pen> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_PEN pen;
 	} W;
 	struct Z {	/* -Z<scale> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double scale;
 		char unit;
 	} Z;
@@ -87,27 +88,27 @@ struct PSWIGGLE_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, nofile = TRUE, paint_wiggle, done;
+	GMT_LONG error = FALSE, nofile = TRUE, paint_wiggle, done;
 
 	char line[BUFSIZ], *units = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], *not_used = NULL;
 
 	GMT_LONG i, j, n, n_alloc = GMT_CHUNK;
-	GMT_LONG k, n_files = 0, fno, n_args, n_expected_fields, n_fields, wantx, wanty;
+	GMT_LONG k, n_files = 0, fno, n_args, n_expected_fields, n_fields, n_read, wantx, wanty;
 
-	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2;
+	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2, d2m = 1.0;
 	double dx, dy, dz, ds, *in, *xx, *yy, *zz, *lon, *lat, *z, start_az, stop_az, fix_az;
 
 	FILE *fp = NULL;
 
 	struct PSWIGGLE_CTRL *Ctrl;
 
-	void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLEAN gave_xy, char *units);
-	void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track);
-	void *New_Pswiggle_Ctrl (), Free_Pswiggle_Ctrl (struct PSWIGGLE_CTRL *C);
+	void GMT_draw_z_scale (double x0, double y0, double length, double zscale, GMT_LONG gave_xy, char *units);
+	void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, GMT_LONG fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, GMT_LONG paint_wiggle, GMT_LONG negative, GMT_LONG outline, GMT_LONG track);
+	void *New_pswiggle_Ctrl (), Free_pswiggle_Ctrl (struct PSWIGGLE_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PSWIGGLE_CTRL *)New_Pswiggle_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct PSWIGGLE_CTRL *)New_pswiggle_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -150,7 +151,12 @@ int main (int argc, char **argv)
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
-					Ctrl->D.value = atof (&argv[i][2]);
+					if (argv[i][2] == 'x') {
+						Ctrl->D.mode = 1;	/* Projected distances (e.g., inches) */
+						Ctrl->D.value = GMT_convert_units (&argv[i][3], GMT_INCH);
+					}
+					else	
+						Ctrl->D.value = atof (&argv[i][2]);	/* In km */
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
@@ -226,7 +232,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"pswiggle %s - Plot xyz-series along tracks\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pswiggle <xyz-files> %s %s -Z<scale>\n", GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[-A<azimuth>] [%s] [-C<center>] [-D<gap>] [%s] [-G<fill>] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_Ho_OPT);
+		fprintf (stderr, "\t[-A<azimuth>] [%s] [-C<center>] [-D[x]<gap>] [%s] [-G<fill>] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_Ho_OPT);
 		fprintf (stderr, "\t[-I<az>] -K [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n");
 		fprintf (stderr, "\t[%s] [-V] [-W<outlinepen>] [%s] [%s]\n\t[%s] [%s] [%s] [%s]] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_mo_OPT);
 
@@ -239,8 +245,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-A set azimuth for preferred positive wiggle orientation [0.0 (north)]\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C sets center value to be removed from z before plotting. [0]\n");
-		fprintf (stderr, "\t-D means there is a datagap if 2 points are more than <gap> distance units apart\n");
-		fprintf (stderr, "\t   gap is in km for map projections, user-units otherwise\n");
+		fprintf (stderr, "\t-D means there is a datagap if 2 points are more than <gap> distance units apart.\n");
+		fprintf (stderr, "\t   For map projections, <gap> is assumed to be in km.\n");
+		fprintf (stderr, "\t   Use -Dx to specify gaps in projected distances (append unit c|i|m|p).\n");
+		fprintf (stderr, "\t   Otherwise, user-units are assumed.\n");
 		GMT_explain_option ('E');
 		GMT_fill_syntax ('G', "Specify color/pattern for positive areas.");
 		GMT_explain_option ('H');
@@ -306,7 +314,10 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	if (GMT_IS_MAPPING) Ctrl->D.value *= 1000.0;    /* Distance in meters */
+	if (GMT_IS_MAPPING) {
+		Ctrl->D.value *= 1000.0;    /* Distance is now in meters */
+		d2m = TWO_PI * project_info.EQ_RAD / 360.0;
+	}
 
 	GMT_plotinit (argc, argv);
 
@@ -393,11 +404,13 @@ int main (int argc, char **argv)
 			}
 			if (GMT_io.status & GMT_IO_EOF) continue;	/* At EOF */
 
-			n = 0;
+			n = n_read = 0;
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
+				n_read++;
+
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
-					exit (EXIT_FAILURE);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
+					continue;
 				}
 
 				lon[n] = in[GMT_X];
@@ -422,17 +435,22 @@ int main (int argc, char **argv)
 			paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && z[0] <= 0.0) || (!Ctrl->N.active && z[0] >= 0.0)));
 			j = 1;
 			for (i = 1; i < n; i++) {	/* Convert to inches/cm and get distance increments */
-				dx = lon[i] - lon[i-1];
-				dy = lat[i] - lat[i-1];
-				if (GMT_IS_MAPPING) {	/* Distance in meters */
-					dx *= cosd (0.5 * (lat[i] + lat[i-1]));
-					ds = hypot (dx, dy) * project_info.M_PR_DEG;
-				}
-				else
-					ds = hypot (dx, dy);
-			
+				
 				GMT_geo_to_xy (lon[i], lat[i], &x_2, &y_2);
 
+				if (Ctrl->D.mode)	/* Check for gap using projected units */
+					ds = hypot (x_2 - xx[j-1], y_2 - yy[j-1]);
+				else {	/* Distance in meters */
+					dx = lon[i] - lon[i-1];
+					dy = lat[i] - lat[i-1];
+					if (GMT_IS_MAPPING) {	/* Distance in meters */
+						dx *= cosd (0.5 * (lat[i] + lat[i-1]));
+						ds = d2m * hypot (dx, dy);
+					}
+					else	/* Plain Cartesian distances of the users coordinates */
+						ds = hypot (dx, dy);
+				}
+			
 				if (j > 0 && (ds > Ctrl->D.value || GMT_is_dnan (z[i]))) {	/* Data gap, plot what we have */
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
 					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
@@ -488,14 +506,14 @@ int main (int argc, char **argv)
 	GMT_free ((void *)yy);
 	GMT_free ((void *)zz);
 
-	Free_Pswiggle_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_pswiggle_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track)
+void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, GMT_LONG fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, GMT_LONG paint_wiggle, GMT_LONG negative, GMT_LONG outline, GMT_LONG track)
 {
 	double dx, dy, len, az = 0.0, s = 0.0, c = 0.0, x_inc, y_inc;
 	GMT_LONG n = 0, i;
@@ -570,7 +588,7 @@ void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, d
 	}
 }
 
-void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLEAN gave_xy, char *units)
+void GMT_draw_z_scale (double x0, double y0, double length, double zscale, GMT_LONG gave_xy, char *units)
 {
 	double dy, off, xx[4], yy[4];
 	char txt[GMT_LONG_TEXT];
@@ -596,10 +614,10 @@ void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLE
 	GMT_text3D (x0 + off, y0, project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], txt, 0.0, 5, 0);
 }
 
-void *New_Pswiggle_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_pswiggle_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSWIGGLE_CTRL *C;
 	
-	C = (struct PSWIGGLE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSWIGGLE_CTRL), "New_Pswiggle_Ctrl");
+	C = (struct PSWIGGLE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSWIGGLE_CTRL), "New_pswiggle_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.value = DBL_MAX;
@@ -612,7 +630,7 @@ void *New_Pswiggle_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void Free_Pswiggle_Ctrl (struct PSWIGGLE_CTRL *C) {	/* Deallocate control structure */
+void Free_pswiggle_Ctrl (struct PSWIGGLE_CTRL *C) {	/* Deallocate control structure */
 	if (C->S.label) free ((void *)C->S.label);	
 	GMT_free ((void *)C);	
 }
