@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psxy.c,v 1.176 2009/07/11 03:16:31 guru Exp $
+ *	$Id: psxy.c,v 1.199 2010/08/26 00:02:07 guru Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -33,48 +33,48 @@
 
 struct PSXY_CTRL {
 	struct A {	/* -A[m|p|step] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;
 		double step;
 	} A;
 	struct C {	/* -C<cpt> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *file;
 	} C;
 	struct D {	/* -D<dx>/<dy> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double dx, dy;
 	} D;
 	struct E {	/* -E[x|X][y|Y][cap][/[+|-]<pen>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG xbar, ybar;	/* 0 = not used, 1 = error bar, 2 = box-whisker, 3 notched box-whisker */
 		GMT_LONG mode;	/* 0 = normal, 1 = -C applies to error pen color, 2 = -C applies to symbol fill & error pen color */
 		double size;
 		struct GMT_PEN pen;
 	} E;
 	struct G {	/* -G<fill> */
-		BOOLEAN active;
+		GMT_LONG active;
 		struct GMT_FILL fill;
 	} G;
 	struct I {	/* -I<intensity> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} I;
 	struct L {	/* -L */
-		BOOLEAN active;
+		GMT_LONG active;
 	} L;
 	struct N {	/* -N */
-		BOOLEAN active;
+		GMT_LONG active;
 	} N;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *arg;
 	} S;
 	struct T {	/* -T */
-		BOOLEAN active;
+		GMT_LONG active;
 	} T;
 	struct W {	/* -W<pen> */
-		BOOLEAN active;
+		GMT_LONG active;
 		GMT_LONG mode;	/* 0 = normal, 1 = -C applies to pen color only, 2 = -C applies to symbol fill & pen color */
 		struct GMT_PEN pen;
 	} W;
@@ -82,13 +82,13 @@ struct PSXY_CTRL {
 
 int main (int argc, char **argv)
 {
-	BOOLEAN	error = FALSE, nofile, polygon = FALSE, read_symbol = FALSE, penset_OK = TRUE, outline_active;
-	BOOLEAN	default_outline = FALSE, done, get_rgb = FALSE, resample, bar_clip;
-	BOOLEAN old_GMT_world_map, error_x = FALSE, error_y = FALSE, def_err_xy = FALSE, clip_set = FALSE, fill_active;
+	GMT_LONG error = FALSE, nofile, polygon = FALSE, read_symbol = FALSE, penset_OK = TRUE, outline_active;
+	GMT_LONG default_outline = FALSE, done, get_rgb = FALSE, resample, bar_clip, save_u = FALSE;
+	GMT_LONG old_GMT_world_map, error_x = FALSE, error_y = FALSE, def_err_xy = FALSE, clip_set = FALSE, fill_active;
 
 	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], *not_used = NULL;
 
-	GMT_LONG i, n, n_alloc = 0, n_total_read = 0, plot_n;
+	GMT_LONG i, n, n_alloc = 0, n_total_read = 0, plot_n, pos2x, pos2y;
 	GMT_LONG j, j0, n_files = 0, fno, xy_errors[2], two, three, four, error_type[2] = {0,0};
 	GMT_LONG n_args, n_fields, n_cols_start = 2, this_outline, change;
 	GMT_LONG n_required, n_expected = 0, reset_cap = 0, error_cols[3] = {1,4,5};
@@ -111,11 +111,11 @@ int main (int argc, char **argv)
 	void plot_y_errorbar (double x, double y, double delta_y, double error_width2, GMT_LONG line);
 	void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG notched);
 	void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG notched);
-	void *New_Psxy_Ctrl (), Free_Psxy_Ctrl (struct PSXY_CTRL *C);
+	void *New_psxy_Ctrl (), Free_psxy_Ctrl (struct PSXY_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 
-	Ctrl = (struct PSXY_CTRL *)New_Psxy_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct PSXY_CTRL *)New_psxy_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	xy_errors[GMT_X] = xy_errors[1] = 0;	/* These will be col # of where to find this info in data */
 
@@ -163,20 +163,19 @@ int main (int argc, char **argv)
 				case 'm':
 				case ':':
 				case '\0':
-					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
+					error += (GMT_LONG)GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
 		
 				/* Supplemental parameters */
 	
 				case 'A':	/* Turn off draw_arc mode */
+					Ctrl->A.active = TRUE;
 					if (argv[i][2] == 'm')
 						Ctrl->A.mode = 1;
 					else if (argv[i][2] == 'p')
 						Ctrl->A.mode = 2;
 					else if (argv[i][2])
 						Ctrl->A.step = atof (&argv[i][2]);	/* Undocumented test feature */
-					else
-						Ctrl->A.active = TRUE;
 					break;
 			
 				case 'C':	/* Vary symbol color with z */
@@ -230,7 +229,7 @@ int main (int argc, char **argv)
 					break;
 				case 'G':		/* Set fill for symbols or polygon */
 					Ctrl->G.active = TRUE;
-					if (!argv[i][2] || (argv[i][2] && GMT_getfill (&argv[i][2], &Ctrl->G.fill))) {
+					if (!argv[i][2] || GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
 						GMT_fill_syntax ('G', " ");
 						error++;
 					}
@@ -313,7 +312,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-S to select symbol type and symbol size (in %s).  Choose between\n", GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t   -(xdash), +(plus), st(a)r, (b|B)ar, (c)ircle, (d)iamond, (e)llipse, (f)ront, octa(g)on, (h)exagon, (i)nvtriangle,\n");
-		fprintf (stderr, "\t   (j)rotated rectangle, (k)ustom, (l)etter, pe(n)tagon, (p)oint, (q)uoted line, (r)ect, (s)quare,\n");
+		fprintf (stderr, "\t   (j)rotated rectangle, (k)ustom, (l)etter, (m)athangle, pe(n)tagon, (p)oint, (q)uoted line, (r)ect, (s)quare,\n");
 		fprintf (stderr, "\t   (t)riangle, (v)ector, (w)edge, (x)cross, (y)dash\n");
 		fprintf (stderr, "\t   If no size is specified, psxy expects the 3rd column to have sizes.\n");
 		fprintf (stderr, "\t   If no symbol is specified, psxy expects the last column to have symbol code.\n");
@@ -322,8 +321,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Bars: Append b<base> to give the y-value of the base [Default = 0]\n");
 		fprintf (stderr, "\t      Append u if width is in x-input units [Default is %s]\n", GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t      Use upper case -SB for horizontal bars (base then refers to x and width may be in y-units [Default is vertical]\n");
-		fprintf (stderr, "\t   Ellipses: the direction, semi-major, and semi-minor axis must be in input columns 3, 4 and 5.\n");
-		fprintf (stderr, "\t     If -SE rather than -Se is selected, psxy will expect azimuth, and semi-axes in km\n");
+		fprintf (stderr, "\t   Ellipses: the direction, major, and minor axis must be in input columns 3, 4 and 5.\n");
+		fprintf (stderr, "\t     If -SE rather than -Se is selected, psxy will expect azimuth, and axes in km\n");
 		fprintf (stderr, "\t     and convert azimuths based on the chosen map projection.  If projection is linear then\n");
 		fprintf (stderr, "\t     we scale the minor/major axes by the map scale.\n");
 		fprintf (stderr, "\t   Rotatable Rectangle: Direction, x-dimension, and y-dimension are expected in columns 3, 4, and 5.\n");
@@ -340,9 +339,11 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t       triangle : diagonal square when centered, directed triangle otherwise.\n"); 
 		fprintf (stderr, "\t     offset = start plotting symbols when along-track distance equals offset [0].\n"); 
 		fprintf (stderr, "\t   Kustom: append <symbolname> immediately after 'k'; this will look for <symbolname>.def in\n");
-		fprintf (stderr, "\t     the current directory or in %s or in %s/custom.\n", GMT_USERDIR, GMT_SHAREDIR);
+		fprintf (stderr, "\t     the current directory, in $GMT_USERDIR, or in $GMT_SHAREDIR (in that order).\n");
 		GMT_list_custom_symbols ();
 		fprintf (stderr, "\t   Letter: append /<string> after symbol size, and optionally %%<font>\n");
+		fprintf (stderr, "\t   Mathangle: the start and stop directions of the math angle must be in input columns 3, 4.\n");
+		fprintf (stderr, "\t     Use -Smf for arrow at first angle, -Sml for last angle, -Smb for both [none].\n");
 		fprintf (stderr, "\t   Quoted line: Give [d|f|n|l|x]<info>[:<labelinfo>]\n");
 		fprintf (stderr, "\t     <code><info> controls placement of labels along lines.  Choose among five algorithms:\n");
 		GMT_cont_syntax (7, 1);
@@ -476,22 +477,35 @@ int main (int argc, char **argv)
 		if (get_rgb) n_cols_start++;
 	}
 
+	/* For most symbols, the data columns beyond two will be dimensions that either have the units appended (e.g., 2c)
+	 * or they are assumed to be in the current measure unit (MEASURE_UNIT).  We therefore set the in_col_type to be
+	 * GMT_IS_DIMENSION for these so that unit conversions are handled correctly.  However, some symbols also require
+	 * angles via the input data file.  S.n_nondim and S.nondim_col are used to reset the in_col_type back to GMT_IS_FLOAT
+	 * for those columns are expected to contain angles.  When NO SYMBOL is specified in -S we must parse the ASCII data
+	 * record to determine the symbol, and this happens AFTER we have already converted the dimensions.  We must therefore
+	 * undo this scaling based on what columns might be angles. */
+	
+	two   = (get_rgb) ? 3 : 2;
+	three = (get_rgb) ? 4 : 3;
+	four  = (get_rgb) ? 5 : 4;
+	pos2x = two + gmtdefs.xy_toggle[GMT_IN];	/* Column with a 2nd longitude (for VECTORS with two sets of coordinates) */
+	pos2y = three - gmtdefs.xy_toggle[GMT_IN];	/* Column with a 2nd latitude (for VECTORS with two sets of coordinates) */
 	if (Ctrl->E.active) {
+		if (S.read_size) GMT_io.in_col_type[two] = GMT_IS_DIMENSION;	/* Must read symbol size from data record */
 		if (xy_errors[GMT_X] && xy_errors[GMT_Y] && error_type[GMT_X] >= 1) xy_errors[GMT_Y] += error_cols[error_type[GMT_X]] - 1;	/* Need 3 or 4 columns for whisker bars */
 		if (def_err_xy && gmtdefs.xy_toggle[GMT_IN]) {	/* -E should be -Eyx */
-			i_swap (xy_errors[GMT_X], xy_errors[GMT_Y]);
-			i_swap (error_type[GMT_X], error_type[GMT_Y]);
+			l_swap (xy_errors[GMT_X], xy_errors[GMT_Y]);
+			l_swap (error_type[GMT_X], error_type[GMT_Y]);
 		}
 		if (xy_errors[GMT_X]) n_cols_start += error_cols[error_type[GMT_X]], error_x = TRUE;
 		if (xy_errors[GMT_Y]) n_cols_start += error_cols[error_type[GMT_Y]], error_y = TRUE;
 		xy_errors[GMT_X] += (S.read_size + get_rgb);	/* Move 0-2 columns over */
 		xy_errors[GMT_Y] += (S.read_size + get_rgb);
 	}
-	two   = (get_rgb) ? 3 : 2;
-	three = (get_rgb) ? 4 : 3;
-	four  = (get_rgb) ? 5 : 4;
+	else	/* Here we have the usual x y [z] [size] [other args] [symbol] record */
+		for (j = n_cols_start; j < 10; j++) GMT_io.in_col_type[j] = GMT_IS_DIMENSION;		/* Since these may have units appended */
+	for (j = 0; j < S.n_nondim; j++) GMT_io.in_col_type[S.nondim_col[j]+get_rgb] = GMT_IS_FLOAT;	/* Since these are angles, not dimensions */
 	n_required = n_cols_start + S.n_required;
-
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_required;
 	n_expected = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : n_required;
 	if (n_expected < n_required) {
@@ -506,12 +520,17 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
+	if (S.u_set) {	/* When -Sc<unit> is given we temporarily reset the system unit to these units so conversions will work */
+		save_u = gmtdefs.measure_unit;
+		gmtdefs.measure_unit = S.u;
+	}
+	
 	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {
 		if (GMT_contlabel_prep (&S.G, NULL)) exit (EXIT_FAILURE);
 		penset_OK = FALSE;	/* The pen for quoted lines are set within the PSL code itself so we dont do it here in psxy */
 	}
 
-	resample = (!Ctrl->A.active && GMT_IS_MAPPING);
+	resample = ((!Ctrl->A.active || Ctrl->A.mode) && GMT_IS_MAPPING);
 	/* Maximum step size (in degrees) used for interpolation of line segments along great circles (if requested) */
 	step = Ctrl->A.step / project_info.x_scale / project_info.M_PR_DEG;
 
@@ -541,10 +560,14 @@ int main (int argc, char **argv)
 	if (S.symbol == GMT_SYMBOL_BARY && !S.base_set && project_info.xyz_projection[GMT_Y] == GMT_LOG10) S.base = south;	/* Default to south level for vertical log10 bars */
 	if (S.symbol == GMT_SYMBOL_BARX && !S.base_set && project_info.xyz_projection[GMT_X] == GMT_LOG10) S.base = west;	/* Default to west level for horizontal log10 bars */
 	bar_clip = ((S.symbol == GMT_SYMBOL_BARX || S.symbol == GMT_SYMBOL_BARY) && !Ctrl->N.active) ? TRUE : FALSE;		/* Bars get clipped instead of skipped */
-	if ((S.symbol == GMT_SYMBOL_VECTOR || S.symbol == GMT_SYMBOL_VECTOR) && S.v_just == 3) {
+	if ((S.symbol == GMT_SYMBOL_VECTOR || S.symbol == GMT_SYMBOL_VECTOR2) && S.v_just == 3) {
 		/* Reading 2nd coordinate so must set column types */
-		GMT_io.in_col_type[two] = GMT_io.in_col_type[GMT_X];
-		GMT_io.in_col_type[three] = GMT_io.in_col_type[GMT_Y];
+		GMT_io.in_col_type[pos2x] = GMT_io.in_col_type[GMT_X];
+		GMT_io.in_col_type[pos2y] = GMT_io.in_col_type[GMT_Y];
+	}
+	else if (S.symbol == GMT_SYMBOL_VECTOR || S.symbol == GMT_SYMBOL_VECTOR2) {
+		/* Reading 2nd coordinate so must set column types */
+		GMT_io.in_col_type[two] = GMT_IS_FLOAT;	/* Direction */
 	}
 	fill_active = Ctrl->G.active;	/* Make copies because we will change the values */
 	outline_active =  Ctrl->W.active;
@@ -585,7 +608,7 @@ int main (int argc, char **argv)
 		if (S.symbol > 0) {	/* symbol part (not counting GMT_SYMBOL_FRONT and GMT_SYMBOL_QUOTED_LINE) */
 
 			GMT_world_map = (S.symbol == GMT_SYMBOL_ELLIPSE && S.convert_angles) ? FALSE : TRUE;
-			if (read_symbol) n_expected = BUFSIZ;
+			if (read_symbol) n_expected = GMT_MAX_COLUMNS;
 			while ((n_fields = GMT_input (fp, &n_expected, &in)) >= 0 && !GMT_REC_IS_EOF) {
 
 				while (GMT_REC_IS_SEG_HEADER && !GMT_REC_IS_EOF) {	/* Process segment headers */
@@ -598,18 +621,18 @@ int main (int argc, char **argv)
 					}
 					if (change & 4) GMT_setpen (&current_pen);
 
-					if (read_symbol) n_expected = BUFSIZ;
+					if (read_symbol) n_expected = GMT_MAX_COLUMNS;
 					n_fields = GMT_input (fp, &n_expected, &in);
 				}
 				if (GMT_REC_IS_EOF) continue;		/* At EOF */
 
-				if (GMT_REC_IS_ERROR && !read_symbol) {
-					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
-					exit (EXIT_FAILURE);
-				}
 				n_total_read++;
+				if (GMT_REC_IS_ERROR && !read_symbol) {
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected, n_total_read);
+					continue;
+				}
 	
-				if (read_symbol) {
+				if (read_symbol) {	/* The symbol was given as last item on an ASCII data record.  Must parse and set parameters */
 					i = strlen (GMT_io.current_record) - 1;
 					GMT_io.current_record[i--] = 0;	/* Chop off the \n */
 					while (GMT_io.current_record[i] && !strchr (" ,\t", (int)GMT_io.current_record[i])) i--;
@@ -617,7 +640,7 @@ int main (int argc, char **argv)
 					n_required = n_cols_start + S.n_required;
 				}
 				if (S.read_size) {
-					S.size_x = in[two] * GMT_u2u[S.u][GMT_INCH];
+					S.size_x = in[two];
 					S.size_x2 = 0.5 * S.size_x;
 				}
 
@@ -631,9 +654,13 @@ int main (int argc, char **argv)
 				}
 				if (GMT_cpt_skip) continue;	/* Chosen cpt file indicates skip for this z */
 		
+				/* If read_symbol is TRUE we must unto the dimension scaling for items that should have no scaling, e.g., angles, or dimensions given in km which must be projected */
+				
+				if (read_symbol) for (j = 0; j < S.n_nondim; j++) in[S.nondim_col[j]+get_rgb] *= GMT_u2u[GMT_INCH][S.u];	/* Undo scales since these are angles or km, not dimensions */
+				
 				if (S.read_vector) {
 					direction = in[two];
-					length = in[three] * GMT_u2u[S.u][GMT_INCH];
+					length = in[three];
 				}
 				else if (S.symbol == GMT_SYMBOL_ELLIPSE && S.convert_angles == 2) {	/* Leave axes in km */
 					direction = in[two];
@@ -647,8 +674,8 @@ int main (int argc, char **argv)
 				}
 				else if (S.symbol == GMT_SYMBOL_ELLIPSE) {
 					direction = in[two];
-					major = in[three] * GMT_u2u[S.u][GMT_INCH];
-					minor = in[four] * GMT_u2u[S.u][GMT_INCH];
+					major = in[three];
+					minor = in[four];
 				}
 				else if (S.symbol == GMT_SYMBOL_ROTATERECT && S.convert_angles == 2) {	/* Leave axes in km */
 					direction = in[two];
@@ -662,20 +689,20 @@ int main (int argc, char **argv)
 				}
 				else if (S.symbol == GMT_SYMBOL_ROTATERECT) {
 					direction = in[two];
-					x_len = in[three+S.read_size] * GMT_u2u[S.u][GMT_INCH];
-					y_len = in[four+S.read_size] * GMT_u2u[S.u][GMT_INCH];
+					x_len = in[three+S.read_size];
+					y_len = in[four+S.read_size];
 				}
-				else if (S.symbol == GMT_SYMBOL_PIE && S.convert_angles) {
-					dir1 = 90.0 - in[two+S.read_size];
-					dir2 = 90.0 - in[three+S.read_size];
+				else if (S.symbol == GMT_SYMBOL_PIE && S.convert_angles == 1) {	/* Got Cartesian azimuths, get dirs */
+					dir2 = 90.0 - in[two+S.read_size];
+					dir1 = 90.0 - in[three+S.read_size];
 				}
-				else if (S.symbol == GMT_SYMBOL_PIE) {
+				else if (S.symbol == GMT_SYMBOL_PIE || S.symbol == GMT_SYMBOL_MANGLE) {
 					dir1 = in[two+S.read_size];
 					dir2 = in[three+S.read_size];
 				}
 				else if (S.symbol == GMT_SYMBOL_RECT) {
-					x_len = in[two+S.read_size] * GMT_u2u[S.u][GMT_INCH];
-					y_len = in[three+S.read_size] * GMT_u2u[S.u][GMT_INCH];
+					x_len = in[two+S.read_size];
+					y_len = in[three+S.read_size];
 				}
 		
 				/* Skip zero-size symbols */
@@ -709,7 +736,7 @@ int main (int argc, char **argv)
 				}
 				else if (!Ctrl->N.active) {
 					GMT_map_outside (in[GMT_X], in[GMT_Y]);
-					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
+					if (GMT_abs (GMT_x_status_new) > 1 || GMT_abs (GMT_y_status_new) > 1) continue;
 				}
 
 				if (GMT_geo_to_xy (in[GMT_X], in[GMT_Y], &plot_x, &plot_y)) continue;	/* NaNs */
@@ -797,6 +824,10 @@ int main (int argc, char **argv)
 						size = (S.equal_area) ? 1.09963611079 * S.size_x : S.size_x;
 						GMT_hexagon (plot_x, plot_y, 0.0, &size, &current_fill, outline_active);
 						break;
+					case GMT_SYMBOL_MANGLE:
+						dim[0] = S.size_x2; dim[1] = dir1; dim[2] = dir2;
+						GMT_matharc (plot_x, plot_y, 0.0, dim, gmtdefs.vector_shape, &current_pen, S.v_double_heads);
+						break;
 					case GMT_SYMBOL_PENTAGON:
 						size = (S.equal_area) ? 1.14948092619 * S.size_x : S.size_x;
 						GMT_pentagon (plot_x, plot_y, 0.0, &size, &current_fill, outline_active);
@@ -854,7 +885,7 @@ int main (int argc, char **argv)
 						else if (S.convert_angles == 1)	/* Cartesian azimuth */
 							direction = 90.0 - direction;
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[two], in[three], &x2, &y2);
+							GMT_geo_to_xy (in[pos2x], in[pos2y], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
 								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
@@ -871,7 +902,7 @@ int main (int argc, char **argv)
 							}
 						}
 						this_outline = (S.v_double_heads) ? outline_active + 8 : outline_active;
-						GMT_vector (plot_x, plot_y, x2, y2, 0.0, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, &current_fill, this_outline);
+						GMT_vector (plot_x, plot_y, x2, y2, 0.0, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, &current_fill, (GMT_LONG)this_outline);
 						break;
 					case GMT_SYMBOL_VECTOR2:
 						if (S.convert_angles == 2) {
@@ -881,7 +912,7 @@ int main (int argc, char **argv)
 						else if (S.convert_angles == 1)	/* Cartesian azimuth */
 							direction = 90.0 - direction;
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[two], in[three], &x2, &y2);
+							GMT_geo_to_xy (in[pos2x], in[pos2y], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
 								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
@@ -902,10 +933,10 @@ int main (int argc, char **argv)
 							v_w = S.v_width * length * S.v_shrink;
 							h_l = S.h_length * length * S.v_shrink;
 							h_w = S.h_width * length * S.v_shrink;
-							GMT_vector (plot_x, plot_y, x2, y2, 0.0, v_w, h_l, h_w, gmtdefs.vector_shape, &current_fill, this_outline);
+							GMT_vector (plot_x, plot_y, x2, y2, 0.0, v_w, h_l, h_w, gmtdefs.vector_shape, &current_fill, (GMT_LONG)this_outline);
 						}
 						else	/* Leave as specified */
-							GMT_vector (plot_x, plot_y, x2, y2, 0.0, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, &current_fill, this_outline);
+							GMT_vector (plot_x, plot_y, x2, y2, 0.0, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, &current_fill, (GMT_LONG)this_outline);
 						break;
 					case GMT_SYMBOL_PIE:
 						if (S.convert_angles == 2) {
@@ -913,6 +944,7 @@ int main (int argc, char **argv)
 							dir1 = tmp;
 							GMT_azim_to_angle (in[GMT_X], in[GMT_Y], 1.0, dir2, &tmp);
 							dir2 = tmp;
+							d_swap (dir1, dir2);
 						}
 						dim[0] = S.size_x2; dim[1] = dir1; dim[2] = dir2;
 						GMT_pie (plot_x, plot_y, 0.0, dim, &current_fill, outline_active);
@@ -931,7 +963,7 @@ int main (int argc, char **argv)
 						GMT_draw_custom_symbol (plot_x, plot_y, 0.0, S.size_x, S.custom, &current_pen, &current_fill, outline_active);
 						break;
 				}
-				if (read_symbol) n_expected = BUFSIZ;
+				if (read_symbol) n_expected = GMT_MAX_COLUMNS;
 			}
 		}
 		else {	/* Line/polygon part */
@@ -1020,6 +1052,8 @@ int main (int argc, char **argv)
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
 
+	if (S.u_set) gmtdefs.measure_unit = save_u;	/* Reset unit */
+
 	if (reset_cap == 1) ps_setlinecap (gmtdefs.ps_line_cap);	/* Reset linecap to default */
 
 	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {
@@ -1042,7 +1076,7 @@ int main (int argc, char **argv)
 		GMT_free ((void *)yy);
 	}
 
-	Free_Psxy_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_psxy_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
@@ -1051,42 +1085,46 @@ int main (int argc, char **argv)
 
 void plot_x_errorbar (double x, double y, double delta_x, double error_width2, GMT_LONG line) {
 	double x_1, x_2, y_1, y_2;
+	GMT_LONG tip1, tip2;
 
+	tip1 = tip2 = (error_width2 > 0.0);
 	GMT_geo_to_xy (x - delta_x, y, &x_1, &y_1);
 	GMT_geo_to_xy (x + delta_x, y, &x_2, &y_2);
 	if (GMT_is_dnan (x_1)) {
 		fprintf (stderr, "%s: Warning: X error bar exceeded domain near line %ld. Set to x_min\n", GMT_program, line);
 		x_1 = project_info.xmin;
+		tip1 = FALSE;
 	}
 	if (GMT_is_dnan (x_2)) {
 		fprintf (stderr, "%s: Warning: X error bar exceeded domain near line %ld. Set to x_max\n", GMT_program, line);
 		x_2 = project_info.xmax;
+		tip2 = FALSE;
 	}
 	ps_segment (x_1, y_1, x_2, y_2);
-	if (error_width2 > 0.0) {
-		ps_segment (x_1, y_1 - error_width2, x_1, y_1 + error_width2);
-		ps_segment (x_2, y_2 - error_width2, x_2, y_2 + error_width2);
-	}
+	if (tip1) ps_segment (x_1, y_1 - error_width2, x_1, y_1 + error_width2);
+	if (tip2) ps_segment (x_2, y_2 - error_width2, x_2, y_2 + error_width2);
 }
 
 void plot_y_errorbar (double x, double y, double delta_y, double error_width2, GMT_LONG line) {
 	double x_1, x_2, y_1, y_2;
+	GMT_LONG tip1, tip2;
 
+	tip1 = tip2 = (error_width2 > 0.0);
 	GMT_geo_to_xy (x, y - delta_y, &x_1, &y_1);
 	GMT_geo_to_xy (x, y + delta_y, &x_2, &y_2);
 	if (GMT_is_dnan (y_1)) {
 		fprintf (stderr, "%s: Warning: Y error bar exceeded domain near line %ld. Set to y_min\n", GMT_program, line);
 		y_1 = project_info.ymin;
+		tip1 = FALSE;
 	}
 	if (GMT_is_dnan (y_2)) {
 		fprintf (stderr, "%s: Warning: Y error bar exceeded domain near line %ld. Set to y_max\n", GMT_program, line);
 		y_2 = project_info.ymax;
+		tip2 = FALSE;
 	}
 	ps_segment (x_1, y_1, x_2, y_2);
-	if (error_width2 > 0.0) {
-		ps_segment (x_1 - error_width2, y_1, x_1 + error_width2, y_1);
-		ps_segment (x_2 - error_width2, y_2, x_2 + error_width2, y_2);
-	}
+	if (tip1) ps_segment (x_1 - error_width2, y_1, x_1 + error_width2, y_1);
+	if (tip2) ps_segment (x_2 - error_width2, y_2, x_2 + error_width2, y_2);
 }
 
 void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG kind) {
@@ -1173,10 +1211,10 @@ void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2,
 	}
 }
 
-void *New_Psxy_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_psxy_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSXY_CTRL *C;
 	
-	C = (struct PSXY_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSXY_CTRL), "New_Psxy_Ctrl");
+	C = (struct PSXY_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSXY_CTRL), "New_psxy_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
@@ -1188,7 +1226,7 @@ void *New_Psxy_Ctrl () {	/* Allocate and initialize a new control structure */
 	return ((void *)C);
 }
 
-void Free_Psxy_Ctrl (struct PSXY_CTRL *C) {	/* Deallocate control structure */
+void Free_psxy_Ctrl (struct PSXY_CTRL *C) {	/* Deallocate control structure */
 	if (C->C.file) free ((void *)C->C.file);	
 	GMT_free ((void *)C);	
 }
