@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: splitxyz.c,v 1.49 2009/07/11 03:16:31 guru Exp $
+ *	$Id: splitxyz.c,v 1.57 2010/05/10 23:12:32 remko Exp $
  *
- *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2010 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -33,41 +33,41 @@
 
 struct SPLITXYZ_CTRL {
 	struct A {	/* -A<azimuth>/<tolerance> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double azimuth, tolerance;
 	} A;
 	struct C {	/* -C<course_change> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} C;
 	struct D {	/* -D<mindist> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} D;
 	struct F {	/* -F<xy_filter>/<z_filter> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double xy_filter, z_filter;
 	} F;
 	struct G {	/* -D<gap> */
-		BOOLEAN active;
+		GMT_LONG active;
 		double value;
 	} G;
 	struct M {	/* -M */
-		BOOLEAN active;
+		GMT_LONG active;
 	} M;
 	struct N {	/* -N<namestem> */
-		BOOLEAN active;
+		GMT_LONG active;
 		char *name;
 	} N;
 	struct Q {	/* -Q[<xyzdg>] */
-		BOOLEAN active;
+		GMT_LONG active;
 		char col[SPLITXYZ_N_OUTPUT_CHOICES];	/* Character codes for desired output in the right order */
 	} Q;
 	struct S {	/* -S */
-		BOOLEAN active;
+		GMT_LONG active;
 	} S;
 	struct Z{	/* -Z */
-		BOOLEAN active;
+		GMT_LONG active;
 	} Z;
 };
 
@@ -79,9 +79,9 @@ struct  DATA    {
 int main(int argc, char **argv)
 {
 	GMT_LONG     i, ndata, n_alloc, nprofiles, begin, end;
-	GMT_LONG	j, k, n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES], n_expected_fields, n_outputs = 0, n_fields;
+	GMT_LONG	j, k, n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES], n_expected_fields, n_outputs = 0, n_read, n_fields;
 
-	BOOLEAN	error = FALSE, ok, hilow = FALSE, z_selected = FALSE;
+	GMT_LONG	error = FALSE, ok, hilow = FALSE, z_selected = FALSE;
 
 	double	dy, dx, last_d, last_c, last_s, csum, ssum, this_c, this_s, dotprod;
 	double	mean_azim, fwork[SPLITXYZ_F_RES], *in, out[SPLITXYZ_N_OUTPUT_CHOICES];
@@ -94,11 +94,11 @@ int main(int argc, char **argv)
 	FILE    *fp = NULL, *fpout;
 
 	void	filterz(struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, GMT_LONG hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, GMT_LONG hilow);
-	void *New_Splitxyz_Ctrl (), Free_Splitxyz_Ctrl (struct SPLITXYZ_CTRL *C);
+	void *New_splitxyz_Ctrl (), Free_splitxyz_Ctrl (struct SPLITXYZ_CTRL *C);
 	
-	argc = GMT_begin (argc, argv);
+	argc = (int)GMT_begin (argc, argv);
 	
-	Ctrl = (struct SPLITXYZ_CTRL *)New_Splitxyz_Ctrl ();	/* Allocate and initialize a new control structure */
+	Ctrl = (struct SPLITXYZ_CTRL *)New_splitxyz_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	fpout = GMT_stdout;
 	memset ((void *)output_choice, 0, SPLITXYZ_N_OUTPUT_CHOICES * sizeof (GMT_LONG));
@@ -115,7 +115,7 @@ int main(int argc, char **argv)
 				case 'b':
 				case 'f':
 				case '\0':
-					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
+					error += (GMT_LONG)GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
 
 				/* Supplemental parameters */
@@ -345,8 +345,7 @@ int main(int argc, char **argv)
 #endif
 	}
         n_alloc = GMT_CHUNK;
-	ndata = 0;
-	i = 0;
+	ndata = n_read = 0;
 
         data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
@@ -354,12 +353,12 @@ int main(int argc, char **argv)
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
+		n_read++;
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, i);
-			exit (EXIT_FAILURE);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields,  n_expected_fields, n_read);
+			continue;
 
 		}
-		i++;
                 if (ndata == n_alloc) {
                         n_alloc <<= 1;
                         data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
@@ -492,8 +491,10 @@ int main(int argc, char **argv)
 							exit (EXIT_FAILURE);
 						}
 					}
-					else
-						fprintf (GMT_stdout, "> Start profile # %ld\n", nprofiles);
+					else {
+						sprintf (buffer, "> Start profile # %ld\n", nprofiles);
+						GMT_fputs (buffer, GMT_stdout);
+					}
 
 					for (i = begin; i < end; i++) {
 						for (j = 0; j < n_outputs; j++) {	/* Remember to convert CCW angles back to azimuths */
@@ -517,7 +518,7 @@ int main(int argc, char **argv)
 	if (gmtdefs.verbose) fprintf(stderr,"%s:  Split %ld data into %ld files.\n", GMT_program, ndata, nprofiles);
 	GMT_free ((void *)data);
 
-	Free_Splitxyz_Ctrl (Ctrl);	/* Deallocate control structure */
+	Free_splitxyz_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
 
@@ -641,10 +642,10 @@ void filterxy (struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter
 	return;
 }
 
-void *New_Splitxyz_Ctrl () {	/* Allocate and initialize a new control structure */
+void *New_splitxyz_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SPLITXYZ_CTRL *C;
 	
-	C = (struct SPLITXYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SPLITXYZ_CTRL), "New_Splitxyz_Ctrl");
+	C = (struct SPLITXYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SPLITXYZ_CTRL), "New_splitxyz_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
         C->A.azimuth = 90.0;
@@ -653,7 +654,7 @@ void *New_Splitxyz_Ctrl () {	/* Allocate and initialize a new control structure
 	return ((void *)C);
 }
 
-void Free_Splitxyz_Ctrl (struct SPLITXYZ_CTRL *C) {	/* Deallocate control structure */
+void Free_splitxyz_Ctrl (struct SPLITXYZ_CTRL *C) {	/* Deallocate control structure */
 	if (C->N.name) free ((void *)C->N.name);	
 	GMT_free ((void *)C);	
 }
